import json
import os
import pathlib
import subprocess
import sys
import tempfile
import xml.etree.ElementTree
from argparse import ArgumentParser

import six


class ExtractCudaInfo:
    def __init__(self, json_file_name, no_std_out):
        self.__json_file_name = json_file_name
        self.__do_print = not no_std_out

    def _extract_info(self, nvidia_smi_folder_name):

        nvidia_exe = "nvidia-smi.exe"
        if nvidia_smi_folder_name:
            found_exe = False
            for root, _, files in os.walk(nvidia_smi_folder_name):
                if nvidia_exe in files:
                    nvidia_exe = os.path.join(root, nvidia_exe)
                    found_exe = True
                    break
            if not found_exe:
                return False

        xml_output = None
        with tempfile.TemporaryDirectory() as tmp_dir:
            xml_output_file_name = pathlib.Path(tmp_dir) / "details.xml"

            p = subprocess.run(
                f'"{nvidia_exe}" -x -q > "{xml_output_file_name}"', shell=True
            )

            if p.returncode != 0:
                return False

            with open(xml_output_file_name) as fh:
                xml_output = fh.read()

        if xml_output:
            xml_root = xml.etree.ElementTree.fromstring(six.ensure_str(xml_output))

            json_dict = {}

            json_dict["driver_version"] = xml_root.find("driver_version").text
            cuda_version_obj = xml_root.find("cuda_version")
            if cuda_version_obj != None:
                json_dict["cuda_version"] = cuda_version_obj.text
            else:
                json_dict["cuda_version"] = "N/A"

            num_gpus = int(xml_root.find("attached_gpus").text)
            json_dict["number_of_gpus"] = num_gpus

            for igpu, gpu in enumerate(xml_root.findall("gpu")):
                gpu_dict = {}

                gpu_dict["product_name"] = gpu.find("product_name").text

                multi_gpu_str = gpu.find("multigpu_board").text
                gpu_dict["is_multi_gpu"] = multi_gpu_str == "Yes"
                board_id_str = gpu.find("board_id").text
                board_id = 0
                if board_id_str == "N/A":
                    board_id = -1
                else:
                    board_id = int(board_id_str, 16)
                gpu_dict["multi_gpu_id"] = board_id

                gpu_mem = int(
                    gpu.find("fb_memory_usage").find("total").text.replace(" MiB", "")
                )
                gpu_dict["total_memory_MB"] = gpu_mem

                json_dict["gpu" + str(igpu)] = gpu_dict

                num_gpus -= 1
            assert num_gpus == 0  # sanity check that all GPUs are processed

            if self.__json_file_name:
                with open(self.__json_file_name, "w") as f:
                    json.dump(json_dict, f, indent=4)

            if self.__do_print:
                print(json.dumps(json_dict, indent=4))

        return True

    def extract(self):
        for folder_name in [
            "C:\\Windows\\System32",
            "C:\\Program Files",
            "C:\\Windows\\System32\\DriverStore\\FileRepository",
            "",
        ]:
            if self._extract_info(folder_name):
                print("Found an nvidia-smi executable")
                return True

        print("No nvidia-smi executable found...")

        if self.__json_file_name:
            with open(self.__json_file_name, "w") as f:
                json.dump({"number_of_gpus": 0}, f, indent=4)

        return False


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument(
        "--json_file", default="", help="Where the output json file shall be stored"
    )
    parser.add_argument(
        "--no_std_out",
        action="store_true",
        default=False,
        help="If something shall be printed to stdout",
    )

    args = parser.parse_args()

    cuda_info = ExtractCudaInfo(args.json_file, args.no_std_out)
    ok = cuda_info.extract()

    if ok:
        sys.exit(0)
    else:
        sys.exit(1)
